<?php

/**
 * @package         TweetMe
 * @version         1.2.1 Free
 * 
 * @author          Tassos Marinos <info@tassos.gr>
 * @link            http://www.tassos.gr
 * @copyright       Copyright © 2016 Tassos Marinos All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

defined('_JEXEC') or die;
jimport( 'joomla.plugin.plugin' );

/**
 *  Novarain Social Share Boxes Class
 */
class NRSocialBoxes extends JPlugin
{

    /**
     *  Application Object
     *
     *  @var  object
     */
    protected $app;

    /**
     *  Plugin short name
     *  Used as html class prefix
     *
     *  @var  string
     */
    protected $shortname = "sb";

    /**
     *  Plugin tag syntax
     *  Used in the curly bracket syntax {plugin}..{/plugin}
     *
     *  @var  string
     */
    protected $tag;

    /**
     *  The loaded indicator of helper
     *
     *  @var  boolean
     */
    protected $init;

    /**
     *  The valid syntax attributes
     *
     *  @var  array
     */
    protected $valid_attributes = array(
        
        "buttontext",
        "url"
    );

    

    /**
     *  Event to add scripts/styles to the document
     *
     *  @return  boolean    True on success
     */
    public function onAfterDispatch()
    {

        if (!$this->getHelper())
        {
            return;
        }

        // Do not load scripts/styles on feed or print pages
        if (!NRFrameworkFunctions::isFeed() && !$this->app->input->getInt('print', 0))
        {

            // Load CSS if enabled
            if ($this->params->get("css", true)) {
                JHtml::stylesheet('plg_system_'.$this->tag.'/'.$this->tag.'.css', array(), true);
            }

            
        }
    }

    /**
     *  Listening to the onContentPrepare event in order to replace module content
     *
     *  @param   string   $context     The context passed to plugin
     *  @param   object   &$row        A JTable object.
     *  @param   array    &$params     The parameters of article or object
     *  @param   integer  $limitstart  
     *
     *  @return  boolean  True on success
     */
    public function onContentPrepare($context, &$row, &$params, $limitstart = 0)
    {
        if (!$this->getHelper() || empty($row->text))
        {
            return;
        }

        if (is_object($row))
        {
            $this->replace($row->text);
        }
        else
        {
            $this->replace($row);
        }

        return true;
    }

    /*
        Listen to the onAfterRender event
    */
    public function onAfterRender()
    {

        if (!$this->getHelper())
        {
            return;
        }

        $html = $this->app->getBody();

        if ($html == '')
        {
            return;
        }

        $this->replace($html);
        $this->clean($html);
        $this->app->setBody($html);
    }

    /**
     * Remove media files from the document if no shortcodes found
     * 
     * Why the hell are we using pre_replace?
     * Well, scripts are injected to the document onAfterDispatch, replaces are taking place onAfterRender so,
     * after the onAfterRender event has taken place there is no other way to remove the media files.
     *
     * We need a better approach.
     *  
     * @param string $html
     */
    private function clean(&$html)
    {
        if (JString::strpos($html, 'class="'.$this->tag) === false)
        {
            $html = preg_replace('#\s*<' . 'link [^>]*href="[^"]*/plg_system_'.$this->tag.'/[^"]*\.css[^"]*"[^>]* />#s', '', $html);
            
        }
    }

    /**
     * Find matched curly bracket syntax
     *
     * @param string $html
     */
    private function find($html)
    {
        // Expression to search for
        $regex = "#{".$this->tag."(.*?)}(.*?){/".$this->tag."}#s";

        // Check whether the plugin should process or not
        if (JString::strpos($html, "{/".$this->tag."}") === false)
        {
            return;
        }

        // Find all instances of the plugin and put them in $matches
        preg_match_all($regex, $html, $matches, PREG_PATTERN_ORDER);

        // Number of matches
        $count = count($matches[0]);

        // Plugin only processes if there are any instances of the plugin in the text
        if (!$count) 
        {
            return;
        }

        return $matches[0];
    }

    /**
    *   Method to replace matches 
    *
    *   @param $html string
    */
    private function replace(&$html)
    {

        $matches = $this->find($html);

        if (!$matches)
        {
            return;
        }

        // If feeds are not allowed remove any instances and exit
        if (NRFrameworkFunctions::isFeed() && !$this->params->get("enableinfeeds", false))
        {
            // Start the replace loop
            foreach ($matches as $key => $match)
            {
                $html = str_replace($match, "", $html);
            }

            return;
        }

        // Default params
        $defaultAttributes = $this->params->toObject();
        $loadedFonts = array();

        // Start the replace loop
        foreach ($matches as $key => $match)
        {

            // Remove multiple white-spaces, tabs and new-lines
            $match = preg_replace('/\s+/S', " ", $match);
            $matchText = preg_replace('@{.+?}@', '', $match);

            $matchAttributes = $this->getAttributes($match);

            // Create a combined attributes JRegistry object
            $attributes = new JRegistry(array_merge((array) $defaultAttributes, (array) $matchAttributes));

            $mode = $attributes->get("mode", "box");

            // Set Theme
            if (!$attributes->get("theme")) {
                $attributes->set("theme", ($mode == "link") ? $attributes->get("linktheme", "bg_yellow") : $attributes->get("boxtheme", "t1_white"));
            }

            $theme = trim($attributes->get("theme"));

            // Set font to inherit in Link Mode
            if ($mode == "link")
            {
                $attributes->set("font", "inherit");
            }

            // Classes
            $classes = array_filter(array(
                $this->shortname.ucfirst($mode),
                !empty($theme) ? $this->shortname."_".$attributes->get("theme") : false,
                $attributes->get("center", false) ? "sbCenter" : false
            ));

            // Create final data object
            $data = new stdClass();
            $data->text   = $matchText;
            $data->params = $attributes;
            $data->class  = implode(" ", $classes);

            if ($mode == "box")
            {
                $maxWidth = $attributes->get("max-width", "650px");

                $styles = array(
                    ($maxWidth != "100%") ? "max-width:" . $maxWidth : null
                );

                $styles = array_filter($styles);

                if (count($styles))
                {
                    $data->styles = "style=\"". implode(";", $styles) . "\"";
                }
            }
            
            $data->url    = $this->generateURL($matchText, $data->params);

            // Create layout
            $layout = new JLayoutFile('default', $basePath = __DIR__ .'/layouts');
            $layoutOutput = $layout->render($data);

            
            
            // Add Custom Code
            if ($key == 0)
            {
                $custom = $this->params->get("customcode", null);

                // Prepare the custom code with content plugins
                if (!empty($custom) && JString::strpos($custom,"{") !== false)
                {
                    $custom = JHtml::_('content.prepare', $custom);
                }
                
                $layoutOutput = $custom . $layoutOutput;
            }

            $html = str_replace($match, $layoutOutput, $html);
        }
    }

    

    /**
     * Generates the final URL
     * 
     * @param   string  $text    The matched element text
     * @param   array   $params  The attributes of matched element
     * @return  string
     */
    protected function generateURL($text, $params)
    {

        if (!$text) 
        {
            return;
        }

        $url = new NRURLs($params->get("url"));

        // Validate passed URL or use the current URL
        if (!$url->validate())
        {
            $url->set(JURI::current());
        }

        // Append any extra params
        $url = $url->appendParams($params->get("urlparams"))->get();

        $URLInfo = new stdClass();
        $URLShorteningService = $params->get("urlservice", false);

        // Check if URL is already cached
        if ((bool) $URLShorteningService)
        {
            $hash  = MD5($URLShorteningService . $url);
            $cache = NRCache::read($hash, true); 

            if ($cache)
            {
                $url = $cache;
            } else {
                // Not found in cache. Shortening will take place in front-end
                $URLInfo->short = $url;
            }
        }

        // Call the child URL method to construct the final share URL
        $method = $this->tag.'URL';
        if (!method_exists($this, $method))
        {
            return;
        }

        $URLInfo->final = $this->$method($url, $text, $params);

        return $URLInfo;
    }

    /**
     *  Removes unwanted characters from a string
     *
     *  @param   string  $text  
     *  @param   array   $characters  
     *
     *  @return  string  Returns a new cleaned string
     */
    protected function makeSafe($text, $characters = array("@", "#", " "))
    {

        if (!$text)
        {
            return;
        }

        $text = trim($text);
        $text = strip_tags($text);
        
        if (is_array($characters) && count($characters) > 0)
        {
            $text = str_replace($characters, "", $text);
        }

        return $text;
    }

    /**
     *  Finds declared attributes in a string. Attributes syntax param1=value|param2=value
     *
     *  @param   text  $string  The text to search for attributes
     *
     *  @return  array          Returns an array with valid attributes only
     */
    private function getAttributes($string)
    {

        if (!$string) 
        {
            return;
        }

        preg_match('@{'.$this->tag.'\s(.*)}@Us', $string, $matches);

        if (!is_array($matches) || count($matches) < 2)
        {
            return;
        }

        if (!isset($matches[1]) || empty($matches[1])) 
        {
            return;
        }

        $attributes = $matches[1];

        // Check if attributes are cached
        $hash = MD5($this->tag . "_" . $attributes);
        if (NRCache::has($hash))
        {
            return NRCache::get($hash);
        }

        // Return if no attributes found
        if (JString::strpos($attributes,"=") === false) 
        {
            return;
        }

        // Split attributes string to array
        $attributes_array = explode("|", $attributes);

        if (!is_array($attributes_array))
        {
            return;
        }

        $atts = new stdClass();

        // Create a new object with the valid attributes only
        foreach ($attributes_array as $key => $value) 
        {

            $value = trim($value);
            $v = explode("=", $value, 2);

            if (!is_array($v))
            {
                continue;
            }

            $attName  = strtolower($v[0]);
            $attValue = trim($v[1], '"');

            if (!in_array($attName, $this->valid_attributes))
            {
                continue;
            }

            $atts->$attName = $attValue;
        }

        $atts = (array) $atts;
        $result = empty($atts) ? false : $atts;

        return NRCache::set($hash, $result);
    }
    
    

    /**
     *  Loads the helper classes of plugin
     *
     *  @return  bool
     */
    private function getHelper()
    {
        // Return if is helper is already loaded
        if ($this->init)
        {
            return true;
        }

        // Return if we are not in frontend
        if (!$this->app->isSite())
        {
            return false;
        }

        // Check if Novarain Framework is installed
        jimport('joomla.filesystem.file');
        if (!JFile::exists(JPATH_PLUGINS . '/system/nrframework/nrframework.php')) 
        {
            return false;
        }

        // Check if Novarain Framework is enabled
        $p = JPluginHelper::getPlugin('system', "nrframework");
        if (!isset($p->name))
        {
            return false;
        }
        
        // Load Novarain Framework Helpers
        require_once JPATH_PLUGINS . '/system/nrframework/helpers/functions.php';
        require_once JPATH_PLUGINS . '/system/nrframework/helpers/urls/urls.php';

        return ($this->init = true);
    }
}
